@@ -775,12 +775,12 @@ setMethod("get_distance_matrix", signature(object="similarityTable"),
             data.table::setcolorder(distance_table,
                                     neworder=c(element_names[1], as.character(distance_table[[element_names[1]]])))
             
-            # Add rownames into the distance table -- I know. Blasphemy.
-            # Otherwise as.dist doesn't function.
-            rownames(distance_table) <- distance_table[[element_names[1]]]
-            distance_table[, (element_names[1]):=NULL]
-            
-            # Create dissimilarity matrix
+            # Convert to dissimilarity matrix by first getting the labels
+            # corresponding to each sample / feature, and then completing the
+            # converted distance table.
+            element_labels <- as.character(distance_table[[element_names[1]]])
+            distance_table <- as.matrix(distance_table[, mget(element_labels)])
+            rownames(distance_table) <- element_labels
             distance_matrix <- stats::as.dist(distance_table)
             
             return(distance_matrix)
