@@ -7,8 +7,11 @@ NULL
 setMethod("show", signature(object="familiarData"),
           function(object){
             
+            # Make sure the data object is updated.
+            object <- update_object(object=object)
+            
             # Create an initial descriptor.
-            data_str <- paste0("A dataset (", object@name, "; v", object@familiar_version, ")")
+            data_str <- paste0("A dataset (", object@name, "; ", .familiar_version_string(object), ")")
             
             # Add the generating ensemble, if available.
             if(length(object@generating_ensemble) > 0){
