@@ -19,6 +19,13 @@ similarity.compute_similarity <- function(x, y, x_categorical, y_categorical, si
                                                           x_categorical=x_categorical,
                                                           y_categorical=y_categorical,
                                                           similarity_metric=similarity_metric))
+  
+  } else if(similarity_metric %in% c("mutual_information")){
+    similarity <- suppressWarnings(similarity.mutual_information(x=x,
+                                                                 y=y,
+                                                                 x_categorical=x_categorical,
+                                                                 y_categorical=y_categorical,
+                                                                 similarity_metric=similarity_metric))
     
   } else if(similarity_metric %in% c("gower", "euclidean", "manhattan", "chebyshev", "cosine", "canberra", "bray_curtis")){
     # Distance-based similarity measures
@@ -37,7 +44,41 @@ similarity.compute_similarity <- function(x, y, x_categorical, y_categorical, si
 
 
 
-similarity.pseudo_r2 <- function(x, y, x_categorical, y_categorical, similarity_metric){
+similarity.pseudo_r2 <- function(x, y, x_categorical, y_categorical, similarity_metric, type="approximate"){
+  
+  ..encode_variable_to_list <- function(x, is_categorical, insert_intercept=FALSE){
+    
+    if(is_categorical) {
+      # Categorical variables are encoded as numeric levels (ordinal), or using
+      # one-hot-encoding (nominal).
+      if(is.ordered(x)){
+        x <- list(as.numeric(x))
+        
+      } else {
+        # Dummy encoding of categorical variable.
+        level_names <- levels(x)
+        level_count <- nlevels(x)
+        
+         x <- lapply(level_names[2:level_count], function(ii, x) (as.numeric(x==ii)), x=x)
+      }
+      
+    } else {
+      # Numeric variables are only stored as a list.
+      x <- list(x)
+    }
+    
+    # Set names
+    names(x) <- paste0("name_", seq_along(x))
+    
+    if(insert_intercept){
+      x <- c(x,
+             list("intercept__"=numeric(length(x)) + 1.0))
+    }
+    
+    return(data.table::as.data.table(x))
+  }
+  
+  .check_parameter_value_is_valid(type, "type", c("exact", "approximate"))
   
   # Check categorical flag for x
   if(length(x_categorical) != 1){
@@ -49,6 +90,36 @@ similarity.pseudo_r2 <- function(x, y, x_categorical, y_categorical, similarity_
     ..error_variable_has_too_many_values(x=y_categorical, var_name="y_categorical", req_length=1)
   }
   
+  # Remove missing elements.
+  valid_elements <- is.finite(x) & is.finite(y)
+  if(sum(valid_elements) <= 1) return(NA_real_)
+  
+  # Keep only valid elements.
+  x <- x[valid_elements]
+  y <- y[valid_elements]
+  
+  # Check if there are more than one unique values in x and or y.
+  if(length(unique(x)) == 1 & length(unique(y)) == 1) return(1.0)
+  
+  if(type == "approximate"){
+    
+    # Select the number of samples for the computing approximate distances.
+    n_samples <- min(c(length(x), ceiling(1000^0.3 * length(x)^0.7)))
+    
+    # Set sample indices - avoid selecting samples randomly, as that is a
+    # somewhat costly operation.
+    sample_index <-  as.integer(round(seq_len(n_samples) / n_samples * length(x)))
+    
+    # Select sample subset.
+    x <- x[sample_index]
+    if(x_categorical) x <- droplevels(x)
+    y <- y[sample_index]
+    if(y_categorical) y <- droplevels(y)
+  }
+  
+  # Check if there are more than one unique values in x and or y.
+  if(length(unique(x)) == 1 & length(unique(y)) == 1) return(1.0)
+  
   # Find analysis type and whether x and y should be swapped in the models,
   # based on information content.
   analysis_info <- similarity.pseudo_r2.get_analysis_type(x=x,
@@ -56,72 +127,175 @@ similarity.pseudo_r2 <- function(x, y, x_categorical, y_categorical, similarity_
                                                           x_categorical=x_categorical,
                                                           y_categorical=y_categorical)
   
-  # Determine model formulae
-  model_formula <- stats::as.formula(ifelse(analysis_info$swap, "x ~ y", "y ~ x"))
-  null_formula <- stats::as.formula(ifelse(analysis_info$swap, "x ~ 1", "y ~ 1"))
-  
-  # Create data.table to perform the regression
-  data <- data.table::data.table("x"=x, "y"=y)
-  
-  # Only allow complete observations. This check should not be required in
-  # practice.
-  data <- data[is.finite(x) & is.finite(y)]
-  
-  # Check if there is sufficient data left over.
-  if(nrow(data) <= 1) return(as.double(NA))
-  
-  # Check if there are more than one unique values in x and or y.
-  if(length(unique(x)) == 1 & length(unique(y)) == 1) return(1.0)
-  
   # Compute log-likelihoods so that the pseudo-R^2 measures can be computed.
   if(analysis_info$type == "gaussian"){
-    # Numerical y variable
-    model_obj <- stats::glm(model_formula, data=data, family=stats::gaussian)
     
-    # Check for models where the intercept completely suffices, i.e. the scale
-    # equals 0. This can happen if one of the variables is invariant.
-    if(!is.finite(model_obj$coefficients[["x"]])) return(0.0)
-    if(near(model_obj$coefficients[["x"]], 0.0, df=2*length(x))) return(0.0)
+    #### Regression ------------------------------------------------------------
     
-    # Check for almost exact copies, which do not show any residual deviance.
-    if(near(model_obj$deviance, 0.0)) return(1.0)
-    
-    null_obj  <- stats::glm(null_formula, data=data, family=stats::gaussian)
+    if(require_package("fastglm", message_type="silent")){
+      
+      if(!analysis_info$swap){
+        predictors <- ..encode_variable_to_list(x=x,
+                                                is_categorical=x_categorical,
+                                                insert_intercept=TRUE)
+        response <- y
+        
+      } else {
+        predictors <- ..encode_variable_to_list(x=y,
+                                                is_categorical=y_categorical,
+                                                insert_intercept=TRUE)
+        response <- x
+      }
+      
+      # Fit informative model.
+      model <- fastglm::fastglm(x=as.matrix(predictors),
+                                y=response,
+                                family=stats::gaussian(),
+                                method=3L)
+      
+      predictor_names <- setdiff(names(model$coefficients), "intercept__")
+      if(any(!is.finite(model$coefficients[predictor_names]))) return(0.0)
+      if(all(approximately(model$coefficients[predictor_names], 0.0))) return(0.0)
+      
+      if(approximately(model$deviance, 0.0)) return(1.0)
+      
+      # Fit uninformative model
+      null_model <- fastglm::fastglm(x=as.matrix(numeric(length(response)), ncol=1L),
+                                     y=response,
+                                     family=stats::gaussian(),
+                                     method=3L)
+      
+    } else {
+      
+      # Determine model formulae
+      model_formula <- stats::as.formula(ifelse(analysis_info$swap, "x ~ y", "y ~ x"))
+      null_formula <- stats::as.formula(ifelse(analysis_info$swap, "x ~ 1", "y ~ 1"))
+      
+      # Numerical y variable
+      model <- stats::glm(model_formula,
+                          data=data.table::data.table("x"=x, "y"=y),
+                          family=stats::gaussian())
+      
+      # Check for models where the intercept completely suffices, i.e. the scale
+      # equals 0. This can happen if one of the variables is invariant.
+      if(!is.finite(model$coefficients[["x"]])) return(0.0)
+      if(near(model$coefficients[["x"]], 0.0, df=2*length(x))) return(0.0)
+      
+      # Check for almost exact copies, which do not show any residual deviance.
+      if(approximately(model$deviance, 0.0)) return(1.0)
+      
+      null_model  <- stats::glm(null_formula,
+                                data=data.table::data.table("x"=x, "y"=y),
+                                family=stats::gaussian())
+    }
     
     # Compute log-likelihoods
-    model_loglik <- stats::logLik(model_obj)[1]
-    null_loglik <- stats::logLik(null_obj)[1]
+    model_loglik <- stats::logLik(model)[1]
+    null_loglik <- stats::logLik(null_model)[1]
     
   } else if(analysis_info$type == "binomial"){
-    # Categorical y variable with two levels
-    model_obj <- stats::glm(model_formula, data=data, family=stats::binomial)
     
-    # Check for models where the intercept completely suffices, i.e. the scale
-    # equals 0. This can happen if one of the variables is invariant.
-    scale_coefficients <- setdiff(names(coef(model_obj)), "(Intercept)")
-    if(all(!is.finite(model_obj$coefficients[scale_coefficients]))) return(0.0)
-    if(all(near(model_obj$coefficients[scale_coefficients], 0.0, df=2*length(x)))) return(0.0)
+    #### Two-class -------------------------------------------------------------
     
-    # Check for almost exact copies, which do not show any residual deviance.
-    if(near(model_obj$deviance, 0.0)) return(1.0)
-    
-    null_obj  <- stats::glm(null_formula, data=data, family=stats::binomial)
+    if(require_package("fastglm", message_type="silent")){
+      
+      if(!analysis_info$swap){
+        predictors <- ..encode_variable_to_list(x=x,
+                                                is_categorical=x_categorical,
+                                                insert_intercept=TRUE)
+        response <- as.numeric(y) - 1
+        
+      } else {
+        predictors <- ..encode_variable_to_list(x=y,
+                                                is_categorical=y_categorical,
+                                                insert_intercept=TRUE)
+        response <- as.numeric(x) - 1
+      }
+      
+      # Fit informative model.
+      model <- fastglm::fastglm(x=as.matrix(predictors),
+                                y=response,
+                                family=stats::binomial(),
+                                method=3L)
+      
+      predictor_names <- setdiff(names(model$coefficients), "intercept__")
+      if(any(!is.finite(model$coefficients[predictor_names]))) return(0.0)
+      if(all(approximately(model$coefficients[predictor_names], 0.0))) return(0.0)
+
+      if(approximately(model$deviance, 0.0)) return(1.0)
+      
+      # Fit uninformative model
+      null_model <- fastglm::fastglm(x=as.matrix(numeric(length(response)), ncol=1L),
+                                     y=response,
+                                     family=stats::binomial(),
+                                     method=3L)
+      
+    } else {
+      # Determine model formulae
+      model_formula <- stats::as.formula(ifelse(analysis_info$swap, "x ~ y", "y ~ x"))
+      null_formula <- stats::as.formula(ifelse(analysis_info$swap, "x ~ 1", "y ~ 1"))
+      
+      # Categorical y variable with two levels
+      model <- stats::glm(model_formula,
+                          data=data.table::data.table("x"=x, "y"=y),
+                          family=stats::binomial())
+      
+      # Check for models where the intercept completely suffices, i.e. the scale
+      # equals 0. This can happen if one of the variables is invariant.
+      predictor_names <- setdiff(names(coef(model)), "(Intercept)")
+      if(any(!is.finite(model$coefficients[predictor_names]))) return(0.0)
+      if(all(near(model$coefficients[predictor_names], 0.0, df=2*length(x)))) return(0.0)
+      
+      # Check for almost exact copies, which do not show any residual deviance.
+      if(approximately(model$deviance, 0.0)) return(1.0)
+      
+      null_model  <- stats::glm(null_formula,
+                                data=data.table::data.table("x"=x, "y"=y),
+                                family=stats::binomial())
+    }
     
     # Compute log-likelihoods
-    model_loglik <- stats::logLik(model_obj)[1]
-    null_loglik <- stats::logLik(null_obj)[1]
+    model_loglik <- stats::logLik(model)[1]
+    null_loglik <- stats::logLik(null_model)[1]
     
   } else if(analysis_info$type == "multinomial") {
-    require_package(x="VGAM",
+    #### Multi-class -----------------------------------------------------------
+    
+    require_package(x="nnet",
                     purpose=paste0("to compute log-likelihood pseudo R2 similarity using the ", similarity_metric, " metric"))
     
-    # Categorical y variable with over two levels
-    model_obj <- VGAM::vglm(model_formula, family=VGAM::multinomial, data=data)
-    null_obj  <- VGAM::vglm(null_formula, family=VGAM::multinomial, data=data)
+    if(!analysis_info$swap){
+      predictors <- ..encode_variable_to_list(x=x, is_categorical=x_categorical)
+      response <- data.table::data.table("response"=y)
+      
+    } else {
+      predictors <- ..encode_variable_to_list(x=y, is_categorical=y_categorical)
+      response <- data.table::data.table("response"=x)
+    }
+    
+    # Determine model formulae
+    model_formula <- stats::reformulate(termlabels=names(predictors), response=quote(response))
+    null_formula <- stats::as.formula("response ~ 1")
+    
+    quiet(model <- nnet::multinom(model_formula,
+                                  data=cbind(predictors, response),
+                                  maxit=ifelse(type=="approximate", 100L, 500L),
+                                  MaxNWts=Inf))
+    
+    model_coefficients <- stats::coef(model)
+    predictor_names <- setdiff(colnames(model_coefficients), "(Intercept)")
+    if(any(!is.finite(model_coefficients[, predictor_names]))) return(0.0)
+    if(all(near(as.vector(model_coefficients[, predictor_names]), 0.0, df=2*length(x)))) return(0.0)
+    
+    if(approximately(model$deviance, 0.0)) return(1.0)
+    
+    quiet(null_model <- nnet::multinom(null_formula,
+                                       data=cbind(predictors, response),
+                                       maxit=ifelse(type=="approximate", 100L, 500L)))
     
     # Compute log-likelihoods
-    model_loglik <- VGAM::logLik.vlm(model_obj)[1]
-    null_loglik <- VGAM::logLik.vlm(null_obj)[1]
+    model_loglik <- stats::logLik(model)[1]
+    null_loglik <- stats::logLik(null_model)[1]
     
   } else {
     ..error_reached_unreachable_code("similarity.pseudo_r2: encountered unknown analysis type")
@@ -177,8 +351,8 @@ similarity.pseudo_r2.get_analysis_type <- function(x, y, x_categorical, y_catego
   # swapped in the analysis.
 
   # Determine number of levels of categorical features.
-  n_x <- ifelse(x_categorical, nlevels(x), length(x))
-  n_y <- ifelse(y_categorical, nlevels(y), length(y))
+  n_x <- ifelse(x_categorical, nlevels(x), data.table::uniqueN(x))
+  n_y <- ifelse(y_categorical, nlevels(y), data.table::uniqueN(y))
   
   if(x_categorical & y_categorical){
     # Case 1: Both x and y are categorical variables.
@@ -210,7 +384,7 @@ similarity.pseudo_r2.get_analysis_type <- function(x, y, x_categorical, y_catego
   } else {
     # Case 4: both x and y are numerical variables.
     analysis_type <- "gaussian"
-    requires_swap <- FALSE
+    requires_swap <- n_x < n_y
   }
   
   return(list("type"=analysis_type, "swap"=requires_swap))
@@ -294,6 +468,32 @@ similarity.correlation <- function(x, y, x_categorical, y_categorical, similarit
 }
 
 
+similarity.mutual_information <- function(x, y, x_categorical, y_categorical, similarity_metric){
+  # Remove missing elements.
+  valid_elements <- is.finite(x) & is.finite(y)
+  if(sum(valid_elements) <= 1) return(NA_real_)
+  
+  # Keep only valid elements.
+  x <- x[valid_elements]
+  y <- y[valid_elements]
+  
+  # Check if there are more than one unique values in x and or y.
+  if(length(unique(x)) == 1 & length(unique(y)) == 1) return(1.0)
+  
+  # Ensure that numeric values are actually encoded as numeric, because praznik
+  # handles integers as categorical variables. This is not the convention used
+  # by familiar.
+  if(!x_categorical && is.integer(x)) x <- as.numeric(x)
+  if(!y_categorical && is.integer(y)) y <- as.numeric(y)
+  
+  require_package(x="praznik",
+                  purpose=paste0("to compute similarity using the ", similarity_metric, " metric"))
+  
+  # Compute normalised mutual information.
+  return(praznik::miScores(x, y, threads=1L) / praznik::jhScores(x, y, threads=1L))
+}
+
+
 similarity.distance_based <- function(x, y, x_categorical, y_categorical, similarity_metric){
   
   # Check categorical feature mask for x.
@@ -361,6 +561,10 @@ similarity.to_distance <- function(x, similarity_metric){
     # Correlation-based metrics
     return(1-abs(x))
     
+  } else if(similarity_metric %in% c("mutual_information")){
+    # Normalised mutual information
+    return(1-x)
+    
   } else {
     # Distance metrics (e.g. Gower's distance). These are natural distances.
     return(x)
@@ -369,6 +573,24 @@ similarity.to_distance <- function(x, similarity_metric){
 
 
 
+similarity.highly_similar <- function(x, similarity_metric){
+  
+  if(similarity_metric %in% c("mcfadden_r2", "cox_snell_r2", "nagelkerke_r2")){
+    return(similarity.to_distance(0.8, similarity_metric=similarity_metric))
+    
+  } else if(similarity_metric %in% c("spearman", "kendall", "pearson")){
+    return(similarity.to_distance(0.9, similarity_metric=similarity_metric))
+  
+  } else if(similarity_metric %in% c("mutual_information")){
+    return(similarity.to_distance(0.45, similarity_metric=similarity_metric))
+    
+  } else {
+    return(0.01)
+  }
+}
+
+
+
 similarity.to_similarity <- function(x, similarity_metric){
   # From distance back to similarity.
   if(similarity_metric %in% c("mcfadden_r2", "cox_snell_r2", "nagelkerke_r2")){
@@ -379,6 +601,10 @@ similarity.to_similarity <- function(x, similarity_metric){
     # Correlation-based metrics. Note that the sign is lost at conversion.
     return(1-abs(x))
     
+  } else if(similarity_metric %in% c("mutual_information")){
+    # Normalised mutual information.
+    return(1-x)
+    
   } else {
     # Distance metrics (e.g. Gower's distance). These do not generally have a
     # similarity representation, and are not converted.
@@ -417,6 +643,9 @@ similarity.message_similarity_metric <- function(similarity_metric){
     
   } else if(similarity_metric == "euclidean"){
     return("Euclidean distance")
+  
+  } else if(similarity_metric == "mutual_information"){
+    return("normalised mutual information")
     
   } else {
     ..error_reached_unreachable_code("similarity.message_similarity_metric: encountered unknown similarity metric")
@@ -444,7 +673,7 @@ similarity.requires_normalisation <- function(similarity_metric){
 .get_available_similarity_metrics <- function(data_type="feature"){
   if(data_type %in% c("feature", "cluster")){
     # Pair-wise comparison between features.
-    return(c("mcfadden_r2", "cox_snell_r2", "nagelkerke_r2", "spearman", "kendall", "pearson"))
+    return(c("mcfadden_r2", "cox_snell_r2", "nagelkerke_r2", "spearman", "kendall", "pearson", "mutual_information"))
     
   } else {
     # Pair-wise comparison between samples.
@@ -459,7 +688,7 @@ similarity.metric_range <- function(similarity_metric, as_distance=FALSE){
   similarity_metric <- gsub(x=similarity_metric, pattern="_trim", replacement="", fixed=TRUE)
   similarity_metric <- gsub(x=similarity_metric, pattern="_winsor", replacement="", fixed=TRUE)
   
-  if(similarity_metric %in% c("gower", "mcfadden_r2", "cox_snell_r2", "nagelkerke_r2")){
+  if(similarity_metric %in% c("gower", "mcfadden_r2", "cox_snell_r2", "nagelkerke_r2", "mutual_information")){
     return(c(0.0, 1.0))
     
   } else if(similarity_metric %in% c("spearman", "kendall", "pearson") & !as_distance){
@@ -477,7 +706,7 @@ similarity.metric_range <- function(similarity_metric, as_distance=FALSE){
 
 similarity.default_is_distance <- function(similarity_metric){
   # Returns if the metric is a distance-metric by default.
-  if(similarity_metric %in% c("mcfadden_r2", "cox_snell_r2", "nagelkerke_r2", "spearman", "kendall", "pearson")){
+  if(similarity_metric %in% c("mcfadden_r2", "cox_snell_r2", "nagelkerke_r2", "spearman", "kendall", "pearson", "mutual_information")){
     return(FALSE)
     
   } else {
