@@ -1,3 +1,30 @@
+test_data_package_installed <- function(outcome_type) {
+  run_test <- TRUE
+
+  data_packages <- list(
+    "survival" = "survival",
+    "multinomial" = "datasets",
+    "binomial" = "MASS",
+    "continuous" = "Ecdat",
+    "count" = "MASS"
+  )
+  
+  if (!rlang::is_installed(data_packages[[outcome_type]])) run_test <- FALSE
+  
+  if (!run_test) {
+    rlang::inform(
+      message = paste0(
+        "Cannot run test because the ", 
+        data_packages[[outcome_type]], 
+        " package is not installed."),
+      class = "familiar_message_inform_no_test"
+    )
+  }
+  
+  return(run_test)
+}
+
+
 test.create_good_data_set <- function(outcome_type, to_data_object=TRUE){
   
   # Suppress NOTES due to non-standard evaluation in data.table
